<?php

/**
 * @file
 * Contains \Drupal\tranc\TrancTranslationManager.
 */

namespace Drupal\tranc;

use Drupal\Core\Language\LanguageInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\StringTranslation\TranslationInterface;
use Drupal\Core\StringTranslation\TranslationManager;
use Drupal\Core\StringTranslation\Translator\TranslatorInterface;

class TrancTranslationManager implements TranslationInterface, TranslatorInterface {

  /**
   * Use this langcode to translate to content language.
   */
  const CONTENT_LANGCODE = '[content]';

  /**
   * @var string
   */
  protected $currentLangcode;

  /**
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * @var \Drupal\Core\StringTranslation\TranslationManager
   */
  protected $translationManager;

  /**
   * TrancTranslationManager constructor.
   *
   * @param \Drupal\Core\StringTranslation\TranslationManager $translationManager
   * @param \Drupal\Core\Language\LanguageManager $languageManager
   */
  public function __construct(TranslationManager $translationManager, LanguageManagerInterface $languageManager) {
    $this->translationManager = $translationManager;
    $this->languageManager = $languageManager;
  }

  /**
   * {@inheritdoc}
   */
  public function getStringTranslation($langcode, $string, $context) {
    return $this->translationManager->getStringTranslation($this->replaceLangcode($langcode), $string, $context);
  }

  /**
   * {@inheritdoc}
   */
  public function translate($string, array $args = [], array $options = []) {
    return $this->translationManager->translate($string, $args, $this->replaceLangcodeInOptions($options));
  }

  /**
   * {@inheritdoc}
   */
  public function translateString(TranslatableMarkup $translated_string) {
    return $this->translationManager->translateString(new TranslatableMarkup(
      $translated_string->getUntranslatedString(),
      $translated_string->getArguments(),
      $this->replaceLangcodeInOptions($translated_string->getOptions())
    ));
  }

  /**
   * {@inheritdoc}
   */
  public function formatPlural($count, $singular, $plural, array $args = [], array $options = []) {
    return $this->translationManager->formatPlural($count, $singular, $plural, $args, $this->replaceLangcodeInOptions($options));
  }

  /**
   * {@inheritdoc}
   */
  public function reset() {
    $this->translationManager->reset();
  }

  /**
   * Sets the default langcode.
   *
   * This is not a method on either interface but LanguageRequestSubscriber
   * calls it, so ¯\_(ツ)_/¯.
   */
  public function setDefaultLangcode($langcode) {
    $this->translationManager->setDefaultLangcode($langcode);
  }

  /**
   * Replace %content% with the current content language.
   *
   * return array
   */
  protected function replaceLangcodeInOptions(array $options) {
    if (isset($options['langcode'])) {
      $options['langcode'] = $this->replaceLangcode($options['langcode']);
    }
    return $options;
  }

  /**
   * @param string $langcode
   *
   * @return string
   */
  protected function replaceLangcode(string $langcode) {
    if ($langcode === self::CONTENT_LANGCODE) {
      $langcode = $this->getCurrentContentLangcode();
    }
    return $langcode;
  }

  /**
   * @return string
   */
  protected function getCurrentContentLangcode() {
    if (!isset($this->currentLangcode)) {
      $this->currentLangcode = $this->languageManager->getCurrentLanguage(LanguageInterface::TYPE_CONTENT)->getId();
    }
    return $this->currentLangcode;
  }

}
